import os, sys, subprocess, os.path
import unitig_query

from .bifrost import rtab_format
from .sortunitigs import sort_unitigs

from .opentsne import compressed_pickle
from .opentsne import tsne_train

from .opentsne_embedding import embedding_classifying

from .unitigs_to_fasta import conv_unitigs_fasta

from .__init__ import __version__

def get_options():
    import argparse

    description = 'mge-cluster: consistent typing of mobile genetic elements'
    parser = argparse.ArgumentParser(description=description,
                                     prog='mge-cluster')

    modeGroup = parser.add_argument_group('Operational mode of mge-cluster')
    mode = modeGroup.add_mutually_exclusive_group(required=True)
    mode.add_argument('--existing',
                        action='store_true',
                        default=False,
                        help='Indicate the folder with the mge-cluster model (unitigs.fasta + embedding.pbz2 + cluster.pbz2)')
    mode.add_argument('--create',
                        action='store_true',
                        default=False,
                        help='Create a new mge-cluster model')

    input = parser.add_argument_group('Input')
    input = input.add_mutually_exclusive_group(required=True)
    input.add_argument('--input',
                    help='File listing the absolute or relative path to the nucleotide sequences (fasta format)',
                    default=None)

    output = parser.add_argument_group('Output')
    output.add_argument('--prefix',
                    default='mge-cluster',
                    help='Filename output prefix'
                         '[default = \'mge-cluster\']')
    output.add_argument('--outdir',
                    default='mge-cluster_results',
                    help='Output folder'
                         '[default = \'mge-cluster_results\']')


    bifrost = parser.add_argument_group('Unitig-caller parameters (--create mode)')
    bifrost.add_argument('--kmer',
                        type=int,
                        default=31,
                        help='K-mer size for graph building/querying '
                             '[default = 31]')

    cluster = parser.add_argument_group('HDBSCAN parameters (--create mode)')
    cluster.add_argument('--min_cluster',
                          type=int,
                          default=10,
                          help='Minimum number of sequences to consider a group as a cluster'
                          '[default = 30]')

    variance = parser.add_argument_group('Threshold Variance (--create mode)')
    variance.add_argument('--variance',
                          type=float,
                          default=0.01,
                          help='Threshold variance to discard uninformative unitigs'
                          '[default = 0.01]')

    tsne = parser.add_argument_group('openTSNE parameters (--create mode)')
    tsne.add_argument('--perplexity',
                          type=int,
                          default=30,
                          help='Perplexity value for openTSNE'
                          '[default = 30]')
    tsne.add_argument('--exaggeration',
                          type=int,
                          default=None,
                          help='Exaggeration value for openTSNE'
                          '[default = None]')

    model = parser.add_argument_group('Existing mge-cluster model (--existing mode)')
    model.add_argument('--model_prefix',
                    default=None,
                    help='Filename output prefix used to generate the model'
                         '[default = \'mge-cluster\']')
    model.add_argument('--model_folder',
                    default=None,
                    help='Folder with the file models'
                         '[default = \'mge-cluster_results\']')



    other = parser.add_argument_group('Other commands')
    other.add_argument('--threads',
                        type=int,
                        default=8,
                        help='Number of threads to use'
                             '[default = 8]')
    other.add_argument('--version', action='version',
                       version='%(prog)s '+__version__)

    return parser.parse_args()

def main():
    options = get_options()
    isExist = os.path.exists(options.outdir)
    if not isExist:
       os.makedirs(options.outdir)


    if options.create == True:
        sys.stdout.write("Operational mode of mge-cluster: --create" + "\n")
        filepath = (options.outdir + '/' + options.prefix + ".rtab")
        if(os.path.exists(filepath) == False):

            unitig_map, input_colour_pref = unitig_query.call_unitigs_build(options.input, options.kmer, True, "NA", options.threads, True, False)

            rtab_format(unitig_map, input_colour_pref, options.prefix + ".rtab", options.outdir)

            sys.stdout.write("Unitigs called!" + "\n")

            unitig_file = options.outdir + '/' + options.prefix + ".rtab"

            sort_unitigs(unitig_file, options.prefix,options.outdir)
        tsne_train(options.prefix,options.outdir, options.min_cluster, options.threads, options.perplexity, options.exaggeration, options.variance)
        conv_unitigs_fasta(options.prefix, options.outdir)

    if options.create == False:
        sys.stderr.write("Operational mode of mge-cluster: --existing" + "\n")

        existing_unitigs = (options.model_folder + '/' + options.model_prefix + "_unitigs.fasta")
        unitig_map, input_colour_pref = unitig_query.call_unitigs_build(options.input, options.kmer, False, existing_unitigs, options.threads, True, False)
        rtab_format(unitig_map, input_colour_pref, options.prefix + "_prediction.rtab", options.outdir)
        existing_unitig_file = options.outdir + '/' + options.prefix + "_prediction.rtab"
        sort_unitigs(existing_unitig_file, options.prefix +  "_prediction", options.outdir)
        embedding_classifying(options.model_prefix,options.model_folder, options.outdir, options.prefix)

if __name__ == "__main__":
    main()
